package view

import SortBy
import androidx.compose.runtime.Composable
import model.Globe
import model.Recipe
import recipeappkmp.shared.generated.resources.Res
import recipeappkmp.shared.generated.resources._10_strawberries
import kotlin.time.Duration

fun List<String>.containsPartial(text:String) : Boolean {
    return  this.any {it.lowercase().contains(text.lowercase())}
}

@Composable
fun getFilteredRecipeList(
    tags : List<String>,
    search : String,
    lockTag : String,
    sortBy: SortBy,
    returnAny: Boolean,
    reverse: Boolean,
) : List<Recipe> {
    val items = getRecipeList()

    var recipes :List<Recipe>

    if(!returnAny)
        recipes = items.filter { it.tags.keys.containsAll(tags) }
    else if(tags.isNotEmpty())
        recipes = findMatchingStrings(items, tags)
    else
        recipes = items

    recipes = recipes.filter {
        it.title.lowercase().contains(search.lowercase()) || it.ingredients.containsPartial(search)
    }

    if (lockTag != "" && lockTag != "SURPRISE")
    {
        recipes = recipes.filter { it.tags.contains(lockTag)}
    }

    recipes = when (sortBy){
        SortBy.Name -> recipes.sortedBy { recipe -> recipe.title }
        SortBy.Time -> recipes.sortedBy { recipe -> (recipe.prepTime + recipe.cookTime)}
        SortBy.Ingredients -> recipes.sortedBy { recipe -> recipe.ingredients.count()}
        SortBy.Missing -> recipes.filter { recipe ->
            recipe.cookTime == Duration.INFINITE || recipe.prepTime == Duration.INFINITE || recipe.servings == "Enough" || recipe.image == Res.drawable._10_strawberries
        }
    }

    if(reverse)
        recipes = recipes.reversed()

    return recipes
}

@Composable
fun getRecipeList() : List<Recipe> {
    return Globe.getAllRecipes()
}

fun findMatchingStrings(list1: List<Recipe>, list2: List<String>): List<Recipe> {
    return list1.filter { str1 -> list2.any { str2 -> str1.tags.contains(str2) } }
}